@@ -17,7 +17,15 @@ from typing import Optional
 
 import pytest
 
-from hypothesis import HealthCheck, Verbosity, assume, given, settings, strategies as st
+from hypothesis import (
+    HealthCheck,
+    Verbosity,
+    assume,
+    errors,
+    given,
+    settings,
+    strategies as st,
+)
 from hypothesis.control import current_build_context
 from hypothesis.database import InMemoryExampleDatabase
 from hypothesis.errors import (
@@ -28,19 +36,18 @@ from hypothesis.errors import (
     Unsatisfiable,
 )
 from hypothesis.internal.compat import int_to_bytes
-from hypothesis.internal.conjecture.data import (
+from hypothesis.internal.conjecture.data import ConjectureData, PrimitiveProvider
+from hypothesis.internal.conjecture.engine import ConjectureRunner
+from hypothesis.internal.conjecture.providers import (
     AVAILABLE_PROVIDERS,
     COLLECTION_DEFAULT_MAX_SIZE,
-    ConjectureData,
-    PrimitiveProvider,
 )
-from hypothesis.internal.conjecture.engine import ConjectureRunner
 from hypothesis.internal.floats import SIGNALING_NAN
 from hypothesis.internal.intervalsets import IntervalSet
 
 from tests.common.debug import minimal
 from tests.common.utils import capture_observations, capture_out
-from tests.conjecture.common import ir_nodes
+from tests.conjecture.common import nodes
 
 
 class PrngProvider(PrimitiveProvider):
@@ -55,12 +62,7 @@ class PrngProvider(PrimitiveProvider):
     def draw_boolean(
         self,
         p: float = 0.5,
-        *,
-        forced: Optional[bool] = None,
-        fake_forced: bool = False,
     ) -> bool:
-        if forced is not None:
-            return forced
         return self.prng.random() < p
 
     def draw_integer(
@@ -70,12 +72,8 @@ class PrngProvider(PrimitiveProvider):
         *,
         weights: Optional[dict[int, float]] = None,
         shrink_towards: int = 0,
-        forced: Optional[int] = None,
-        fake_forced: bool = False,
     ) -> int:
         assert isinstance(shrink_towards, int)  # otherwise ignored here
-        if forced is not None:
-            return forced
 
         if weights is not None:
             assert min_value is not None
@@ -102,12 +100,7 @@ class PrngProvider(PrimitiveProvider):
         max_value: float = math.inf,
         allow_nan: bool = True,
         smallest_nonzero_magnitude: float,
-        forced: Optional[float] = None,
-        fake_forced: bool = False,
     ) -> float:
-        if forced is not None:
-            return forced
-
         if allow_nan and self.prng.random() < 1 / 32:
             nans = [math.nan, -math.nan, SIGNALING_NAN, -SIGNALING_NAN]
             return self.prng.choice(nans)
@@ -139,11 +132,7 @@ class PrngProvider(PrimitiveProvider):
         *,
         min_size: int = 0,
         max_size: int = COLLECTION_DEFAULT_MAX_SIZE,
-        forced: Optional[str] = None,
-        fake_forced: bool = False,
     ) -> str:
-        if forced is not None:
-            return forced
         size = self.prng.randint(
             min_size, max(min_size, min(100 if max_size is None else max_size, 100))
         )
@@ -153,12 +142,7 @@ class PrngProvider(PrimitiveProvider):
         self,
         min_size: int = 0,
         max_size: int = COLLECTION_DEFAULT_MAX_SIZE,
-        *,
-        forced: Optional[bytes] = None,
-        fake_forced: bool = False,
     ) -> bytes:
-        if forced is not None:
-            return forced
         max_size = 100 if max_size is None else max_size
         size = self.prng.randint(min_size, max_size)
         try:
@@ -267,7 +251,7 @@ def test_backend_can_shrink_floats():
 
 # mostly a shoehorned coverage test until the shrinker is migrated to the ir
 # and calls cached_test_function_ir with backends consistently.
-@given(ir_nodes())
+@given(nodes())
 def test_new_conjecture_data_ir_with_backend(node):
     def test(data):
         getattr(data, f"draw_{node.ir_type}")(**node.kwargs)
@@ -595,3 +579,20 @@ def test_notes_incorrect_verification(provider):
         with pytest.raises(AssertionError) as ctx:
             test_function()
         assert (msg in ctx.value.__notes__) == (provider is UnsoundVerifierProvider)
+
+
+def test_invalid_provider_kw():
+    with pytest.raises(InvalidArgument, match="got an instance instead"):
+        ConjectureData(
+            random=None,
+            provider=TrivialProvider(None),
+            provider_kw={"one": "two"},
+        )
+
+
+def test_available_providers_deprecation():
+    with pytest.warns(errors.HypothesisDeprecationWarning):
+        from hypothesis.internal.conjecture.data import AVAILABLE_PROVIDERS  # noqa
+
+    with pytest.raises(ImportError):
+        from hypothesis.internal.conjecture.data import does_not_exist  # noqa
