@@ -15,12 +15,8 @@ import pytest
 
 from hypothesis import HealthCheck, assume, given, settings
 from hypothesis.errors import Flaky
-from hypothesis.internal.conjecture.data import (
-    ConjectureData,
-    NodeTemplate,
-    Status,
-    StopTest,
-)
+from hypothesis.internal.conjecture.choice import ChoiceTemplate
+from hypothesis.internal.conjecture.data import ConjectureData, Status, StopTest
 from hypothesis.internal.conjecture.datatree import (
     Branch,
     DataTree,
@@ -36,8 +32,8 @@ from tests.conjecture.common import (
     fresh_data,
     integer_kwargs,
     interesting_origin,
-    ir_nodes,
     kwargs_strategy,
+    nodes,
     run_to_nodes,
 )
 
@@ -128,20 +124,12 @@ def test_novel_prefixes_are_novel():
     runner = ConjectureRunner(tf, settings=TEST_SETTINGS, random=Random(0))
     for _ in range(100):
         prefix = runner.tree.generate_novel_prefix(runner.random)
-        extension = prefix + (NodeTemplate("simplest", size=100),)
+        extension = prefix + (ChoiceTemplate("simplest", count=100),)
         assert runner.tree.rewrite(extension)[1] is None
         result = runner.cached_test_function_ir(extension)
         assert runner.tree.rewrite(extension)[0] == result.choices
 
 
-def test_overruns_if_not_enough_bytes_for_block():
-    runner = ConjectureRunner(
-        lambda data: data.draw_bytes(2, 2), settings=TEST_SETTINGS, random=Random(0)
-    )
-    runner.cached_test_function_ir((b"\0\0",))
-    assert runner.tree.rewrite((b"\0",))[1] is Status.OVERRUN
-
-
 def test_overruns_if_prefix():
     runner = ConjectureRunner(
         lambda data: [data.draw_boolean() for _ in range(2)],
@@ -571,7 +559,7 @@ def _draw(data, node, *, forced=None):
     return getattr(data, f"draw_{node.ir_type}")(**node.kwargs, forced=forced)
 
 
-@given(ir_nodes(was_forced=True, ir_types=["float"]))
+@given(nodes(was_forced=True, ir_types=["float"]))
 def test_simulate_forced_floats(node):
     tree = DataTree()
 
@@ -583,4 +571,4 @@ def test_simulate_forced_floats(node):
     data = ConjectureData.for_choices([node.value], observer=tree.new_observer())
     tree.simulate_test_function(data)
     data.freeze()
-    assert data.ir_nodes == (node,)
+    assert data.nodes == (node,)
