@@ -28,16 +28,15 @@ from hypothesis import (
 from hypothesis.database import (
     ExampleDatabase,
     InMemoryExampleDatabase,
-    ir_from_bytes,
-    ir_to_bytes,
+    choices_from_bytes,
+    choices_to_bytes,
 )
 from hypothesis.errors import FailedHealthCheck, FlakyStrategyDefinition
 from hypothesis.internal.compat import PYPY, bit_count, int_from_bytes
 from hypothesis.internal.conjecture import engine as engine_module
-from hypothesis.internal.conjecture.data import ConjectureData, Overrun, Status, ir_size
+from hypothesis.internal.conjecture.data import ConjectureData, Overrun, Status
 from hypothesis.internal.conjecture.datatree import compute_max_children
 from hypothesis.internal.conjecture.engine import (
-    BUFFER_SIZE_IR,
     MIN_TEST_CALLS,
     ConjectureRunner,
     ExitReason,
@@ -57,7 +56,7 @@ from tests.conjecture.common import (
     TEST_SETTINGS,
     buffer_size_limit,
     interesting_origin,
-    ir_nodes,
+    nodes,
     run_to_nodes,
     shrinking_from,
 )
@@ -88,7 +87,7 @@ def test_can_load_data_from_a_corpus():
     key = b"hi there"
     db = ExampleDatabase()
     value = b"=\xc3\xe4l\x81\xe1\xc2H\xc9\xfb\x1a\xb6bM\xa8\x7f"
-    db.save(key, ir_to_bytes([value]))
+    db.save(key, choices_to_bytes([value]))
 
     def f(data):
         if data.draw_bytes() == value:
@@ -216,7 +215,7 @@ def test_can_navigate_to_a_valid_example():
         data.mark_interesting()
 
     runner = ConjectureRunner(f, settings=settings(max_examples=5000, database=None))
-    with buffer_size_limit(2):
+    with buffer_size_limit(4):
         runner.run()
     assert runner.interesting_examples
 
@@ -319,7 +318,7 @@ def test_reuse_phase_runs_for_max_examples_if_generation_is_disabled():
     with deterministic_PRNG():
         db = InMemoryExampleDatabase()
         for i in range(256):
-            db.save(b"key", ir_to_bytes([i]))
+            db.save(b"key", choices_to_bytes([i]))
         seen = set()
 
         def test(data):
@@ -590,32 +589,14 @@ def test_clears_out_its_database_on_shrinking(
 
     for n in range(256):
         if n != 127 or not skip_target:
-            db.save(runner.secondary_key, ir_to_bytes([n]))
+            db.save(runner.secondary_key, choices_to_bytes([n]))
     runner.run()
     assert len(runner.interesting_examples) == 1
     for b in db.fetch(runner.secondary_key):
-        assert ir_from_bytes(b)[0] >= 127
+        assert choices_from_bytes(b)[0] >= 127
     assert len(list(db.fetch(runner.database_key))) == 1
 
 
-def test_detects_too_small_block_starts():
-    call_count = 0
-
-    def f(data):
-        nonlocal call_count
-        call_count += 1
-        data.draw_bytes(8, 8)
-        data.mark_interesting()
-
-    runner = ConjectureRunner(f, settings=settings(database=None))
-    r = runner.cached_test_function_ir((bytes(8),))
-    assert r.status is Status.INTERESTING
-    assert call_count == 1
-    r2 = runner.cached_test_function_ir((bytes([255] * 7),))
-    assert r2.status is Status.OVERRUN
-    assert call_count == 1
-
-
 def test_shrinks_both_interesting_examples(monkeypatch):
     def generate_new_examples(self):
         self.cached_test_function_ir((1,))
@@ -707,7 +688,9 @@ def test_discarding_can_fail():
         data.mark_interesting()
 
     shrinker.remove_discarded()
-    assert any(e.discarded and e.ir_length > 0 for e in shrinker.shrink_target.examples)
+    assert any(
+        e.discarded and e.choice_count > 0 for e in shrinker.shrink_target.examples
+    )
 
 
 def test_shrinking_from_mostly_zero(monkeypatch):
@@ -768,7 +751,7 @@ def test_database_clears_secondary_key():
     )
 
     for i in range(10):
-        database.save(runner.secondary_key, ir_to_bytes([i]))
+        database.save(runner.secondary_key, choices_to_bytes([i]))
 
     runner.cached_test_function_ir((10,))
     assert runner.interesting_examples
@@ -800,7 +783,7 @@ def test_database_uses_values_from_secondary_key():
         database_key=key,
     )
     for i in range(10):
-        database.save(runner.secondary_key, ir_to_bytes([i]))
+        database.save(runner.secondary_key, choices_to_bytes([i]))
 
     runner.cached_test_function_ir((10,))
     assert runner.interesting_examples
@@ -810,9 +793,9 @@ def test_database_uses_values_from_secondary_key():
     runner.clear_secondary_key()
 
     assert len(set(database.fetch(key))) == 1
-    assert {ir_from_bytes(b)[0] for b in database.fetch(runner.secondary_key)} == set(
-        range(6, 11)
-    )
+    assert {
+        choices_from_bytes(b)[0] for b in database.fetch(runner.secondary_key)
+    } == set(range(6, 11))
 
     (v,) = runner.interesting_examples.values()
     assert v.choices == (5,)
@@ -999,8 +982,8 @@ def test_branch_ending_in_write():
         if count > 1:
             data.draw_boolean(forced=False)
 
-        assert data.ir_nodes not in seen
-        seen.add(data.ir_nodes)
+        assert data.nodes not in seen
+        seen.add(data.nodes)
 
     with deterministic_PRNG():
         runner = ConjectureRunner(tf, settings=TEST_SETTINGS)
@@ -1027,26 +1010,25 @@ SMALL_COUNT_SETTINGS = settings(TEST_SETTINGS, max_examples=500)
 
 
 def test_discards_kill_branches():
-    starts = set()
+    seen = set()
 
-    with deterministic_PRNG():
+    def test(data: ConjectureData):
+        data.start_example(1)
+        n1 = data.draw_integer(0, 9)
+        data.stop_example(discard=n1 > 0)
+        n2 = data.draw_integer(0, 9)
+        n3 = data.draw_integer(0, 9)
 
-        def test(data):
-            assert runner.call_count <= 256
-            while True:
-                data.start_example(1)
-                b = data.draw_integer(0, 2**8 - 1)
-                data.stop_example(discard=b != 0)
-                if len(data.buffer) == 1:
-                    s = bytes(data.buffer)
-                    assert s not in starts
-                    starts.add(s)
-                if b == 0:
-                    break
+        assert (n1, n2, n3) not in seen
+        seen.add((n1, n2, n3))
 
-        runner = ConjectureRunner(test, settings=SMALL_COUNT_SETTINGS)
-        runner.run()
-        assert runner.call_count == 256
+    runner = ConjectureRunner(test, settings=SMALL_COUNT_SETTINGS)
+    runner.run()
+    assert runner.exit_reason is ExitReason.finished
+    # 10 to explore the initial n1 = 0-9 statespace, then 100 to explore just
+    # the (0, n1, n2) statespace, because every prefix other than 0 is a discard
+    # and is not explored. Subtract the overlap, which occurs once at n1 = 0.
+    assert len(seen) == 100 + 10 - 1
 
 
 @pytest.mark.parametrize("n", range(1, 32))
@@ -1063,14 +1045,13 @@ def test_number_of_examples_in_integer_range_is_bounded(n):
 
 def test_prefix_cannot_exceed_buffer_size(monkeypatch):
     buffer_size = 10
-    monkeypatch.setattr(engine_module, "BUFFER_SIZE", buffer_size)
 
-    with deterministic_PRNG():
+    with deterministic_PRNG(), buffer_size_limit(buffer_size):
 
         def test(data):
             while data.draw_boolean():
-                assert data.length_ir <= buffer_size
-            assert data.length_ir <= buffer_size
+                assert data.length <= buffer_size
+            assert data.length <= buffer_size
 
         runner = ConjectureRunner(test, settings=SMALL_COUNT_SETTINGS)
         runner.run()
@@ -1460,62 +1441,44 @@ def test_does_result_for_reuse():
         d1 = runner.cached_test_function_ir((b"\0"), extend=8)
         d2 = runner.cached_test_function_ir(d1.choices)
         assert d1.status == d2.status == Status.VALID
-        assert d1.ir_nodes == d2.ir_nodes
+        assert d1.nodes == d2.nodes
         assert calls == 1
 
 
-def test_does_not_cache_overrun_if_extending():
+def test_does_not_use_cached_overrun_if_extending():
     def test(data):
-        data.draw_bytes(8, 8)
+        data.draw_integer()
+        data.draw_integer()
 
     with deterministic_PRNG():
         runner = ConjectureRunner(test, settings=TEST_SETTINGS)
 
-        d1 = runner.cached_test_function_ir((b"",), extend=4)
-        d2 = runner.cached_test_function_ir((b"",), extend=8)
-        assert d1.status == Status.OVERRUN
-        assert d2.status == Status.VALID
+        data = runner.cached_test_function_ir((1,))
+        assert data.status == Status.OVERRUN
+        assert runner.call_count == 1
+
+        # the choice sequence of (1,) maps to an overrun in the cache, but we
+        # do not want to use this cache entry if we're extending.
+        data = runner.cached_test_function_ir((1,), extend=1)
+        assert data.status == Status.VALID
+        assert runner.call_count == 2
 
 
-def test_does_cache_overrun_if_not_extending():
+def test_uses_cached_overrun_if_not_extending():
     def test(data):
-        data.draw_bytes(8, 8)
-        data.draw_bytes(8, 8)
+        data.draw_integer()
+        data.draw_integer()
 
     with deterministic_PRNG():
         runner = ConjectureRunner(test, settings=TEST_SETTINGS)
 
-        data = runner.cached_test_function_ir((bytes(8)), extend=0)
+        data = runner.cached_test_function_ir((1,), extend=0)
         assert data.status is Status.OVERRUN
+        assert runner.call_count == 1
 
-        data = runner.cached_test_function_ir((bytes(8)), extend=10)
-        assert data.status is Status.VALID
-
-
-def test_does_not_cache_extended_prefix_if_overrun():
-    def test(data):
-        data.draw_bytes(8, 8)
-
-    with deterministic_PRNG():
-        runner = ConjectureRunner(test, settings=TEST_SETTINGS)
-
-        d1 = runner.cached_test_function_ir((b"",), extend=4)
-        d2 = runner.cached_test_function_ir((b"",), extend=8)
-        assert d1.status is Status.OVERRUN
-        assert d2.status is Status.VALID
-
-
-def test_draw_bits_partly_from_prefix_and_partly_random():
-    # a draw_bits call which straddles the end of our prefix has a slightly
-    # different code branch.
-    def test(data):
-        # float consumes draw_bits(64)
-        data.draw_float()
-
-    with deterministic_PRNG():
-        runner = ConjectureRunner(test, settings=TEST_SETTINGS)
-        d = runner.cached_test_function(bytes(10), extend=100)
-        assert d.status == Status.VALID
+        data = runner.cached_test_function_ir((1,), extend=0)
+        assert data.status is Status.OVERRUN
+        assert runner.call_count == 1
 
 
 def test_can_be_set_to_ignore_limits():
@@ -1563,7 +1526,7 @@ def _draw(cd, node):
     return getattr(cd, f"draw_{node.ir_type}")(**node.kwargs)
 
 
-@given(ir_nodes(was_forced=False))
+@given(nodes(was_forced=False))
 def test_overruns_with_extend_are_not_cached(node):
     assume(compute_max_children(node.ir_type, node.kwargs) > 100)
 
@@ -1584,9 +1547,7 @@ def test_overruns_with_extend_are_not_cached(node):
     assert data.status is Status.OVERRUN
 
     # cache miss
-    data = runner.cached_test_function_ir(
-        [node.value], extend=BUFFER_SIZE_IR - ir_size([node])
-    )
+    data = runner.cached_test_function_ir([node.value], extend="full")
     assert runner.call_count == 2
     assert data.status is Status.VALID
 
@@ -1698,7 +1659,7 @@ def test_skips_secondary_if_interesting_is_found():
     for i in range(256):
         db.save(
             runner.database_key if i < 10 else runner.secondary_key,
-            ir_to_bytes([i]),
+            choices_to_bytes([i]),
         )
     runner.reuse_existing_examples()
     assert runner.call_count == 10
@@ -1721,12 +1682,12 @@ def test_discards_invalid_db_entries(key_name):
             database_key=b"stuff",
         )
         key = getattr(runner, key_name)
-        valid = ir_to_bytes([1])
+        valid = choices_to_bytes([1])
         db.save(key, valid)
         for n in range(5):
             b = bytes([255, n])
             # save a bunch of invalid entries under the database key
-            assert ir_from_bytes(b) is None
+            assert choices_from_bytes(b) is None
             db.save(key, b)
 
         assert len(set(db.fetch(key))) == 6
@@ -1753,7 +1714,7 @@ def test_discards_invalid_db_entries_pareto():
         )
         for n in range(5):
             b = bytes([255, n])
-            assert ir_from_bytes(b) is None
+            assert choices_from_bytes(b) is None
             db.save(runner.pareto_key, b)
 
         assert len(set(db.fetch(runner.pareto_key))) == 5
