@@ -13,14 +13,11 @@ import sys
 import pytest
 
 from hypothesis import HealthCheck, assume, example, given, settings, strategies as st
-from hypothesis.internal.compat import ceil, extract_bits, floor, int_to_bytes
+from hypothesis.internal.compat import ceil, extract_bits, floor
 from hypothesis.internal.conjecture import floats as flt
-from hypothesis.internal.conjecture.data import ConjectureData
 from hypothesis.internal.conjecture.engine import ConjectureRunner
 from hypothesis.internal.floats import float_to_int
 
-from tests.conjecture.common import ir
-
 EXPONENTS = list(range(flt.MAX_EXPONENT + 1))
 assert len(EXPONENTS) == 2**11
 
@@ -132,21 +129,18 @@ def float_runner(start, condition, *, kwargs=None):
             data.mark_interesting()
 
     runner = ConjectureRunner(test_function)
-    runner.cached_test_function_ir(ir((float(start), kwargs)))
+    runner.cached_test_function_ir((float(start),))
     assert runner.interesting_examples
     return runner
 
 
 def minimal_from(start, condition, *, kwargs=None):
-    kwargs = {} if kwargs is None else kwargs
-
     runner = float_runner(start, condition, kwargs=kwargs)
     runner.shrink_interesting_examples()
     (v,) = runner.interesting_examples.values()
-    data = ConjectureData.for_ir_tree(v.ir_nodes)
-    result = data.draw_float(**kwargs)
-    assert condition(result)
-    return result
+    f = v.choices[0]
+    assert condition(f)
+    return f
 
 
 INTERESTING_FLOATS = [0.0, 1.0, 2.0, sys.float_info.max, float("inf"), float("nan")]
@@ -201,17 +195,6 @@ def test_does_not_shrink_across_one():
     assert minimal_from(1.1, lambda x: x == 1.1 or 0 < x < 1) == 1.1
 
 
-@pytest.mark.parametrize("f", [2.0, 10000000.0])
-def test_converts_floats_to_integer_form(f):
-    assert flt.is_simple(f)
-    buf = int_to_bytes(flt.base_float_to_lex(f), 8)
-
-    runner = float_runner(f, lambda g: g == f)
-    runner.shrink_interesting_examples()
-    (v,) = runner.interesting_examples.values()
-    assert v.buffer[:-1] < buf
-
-
 def test_reject_out_of_bounds_floats_while_shrinking():
     # coverage test for rejecting out of bounds floats while shrinking
     kwargs = {"min_value": 103.0}
