@@ -25,6 +25,7 @@ from hypothesis import (
 )
 from hypothesis.errors import StopTest
 from hypothesis.internal.conjecture.choice import (
+    ChoiceTemplate,
     choice_equal,
     choice_from_index,
     choice_permitted,
@@ -35,19 +36,15 @@ from hypothesis.internal.conjecture.data import (
     COLLECTION_DEFAULT_MAX_SIZE,
     ConjectureData,
     IRNode,
-    NodeTemplate,
     Status,
-    ir_size,
+    choices_size,
 )
 from hypothesis.internal.conjecture.datatree import (
     MAX_CHILDREN_EFFECTIVELY_INFINITE,
     all_children,
     compute_max_children,
 )
-from hypothesis.internal.conjecture.engine import (
-    BUFFER_SIZE_IR,
-    truncate_choices_to_size,
-)
+from hypothesis.internal.conjecture.engine import choice_count
 from hypothesis.internal.floats import SMALLEST_SUBNORMAL, next_down, next_up
 from hypothesis.internal.intervalsets import IntervalSet
 
@@ -59,8 +56,8 @@ from tests.conjecture.common import (
     fresh_data,
     integer_kw,
     integer_kwargs,
-    ir_nodes,
     ir_types_and_kwargs,
+    nodes,
 )
 
 
@@ -235,7 +232,7 @@ def test_compute_max_children_unbounded_integer_ranges(kwargs):
 
 
 @given(st.randoms())
-def test_ir_nodes(random):
+def test_nodes(random):
     data = fresh_data(random=random)
     data.draw_float(min_value=-10.0, max_value=10.0, forced=5.0)
     data.draw_boolean(forced=True)
@@ -276,10 +273,10 @@ def test_ir_nodes(random):
         ),
         IRNode(ir_type="integer", value=50, kwargs=integer_kw(0, 100), was_forced=True),
     )
-    assert data.ir_nodes == expected_tree_nodes
+    assert data.nodes == expected_tree_nodes
 
 
-@given(ir_nodes())
+@given(nodes())
 def test_copy_ir_node(node):
     assert node == node
 
@@ -291,14 +288,14 @@ def test_copy_ir_node(node):
     )
 
 
-@given(ir_nodes())
+@given(nodes())
 def test_ir_node_equality(node):
     assert node == node
     # for coverage on our NotImplemented return, more than anything.
     assert node != 42
 
 
-@given(ir_nodes(was_forced=True))
+@given(nodes(was_forced=True))
 def test_cannot_modify_forced_nodes(node):
     with pytest.raises(AssertionError):
         node.copy(with_value=42)
@@ -312,7 +309,7 @@ def test_data_with_empty_ir_tree_is_overrun():
     assert data.status is Status.OVERRUN
 
 
-@given(ir_nodes(was_forced=True))
+@given(nodes(was_forced=True))
 def test_data_with_changed_forced_value(node):
     # we had a forced node and then tried to draw a different forced value from it.
     # ir tree: v1 [was_forced=True]
@@ -321,7 +318,7 @@ def test_data_with_changed_forced_value(node):
     # This is actually fine; we'll just ignore the forced node (v1) and return
     # what the draw expects (v2).
 
-    data = ConjectureData.for_choices([node.value], max_length=BUFFER_SIZE_IR)
+    data = ConjectureData.for_choices([node.value])
 
     draw_func = getattr(data, f"draw_{node.ir_type}")
     kwargs = deepcopy(node.kwargs)
@@ -364,7 +361,7 @@ def test_data_with_changed_forced_value(node):
         was_forced=True,
     )
 )
-@given(ir_nodes(was_forced=True))
+@given(nodes(was_forced=True))
 def test_data_with_same_forced_value_is_valid(node):
     # we had a forced node and then drew the same forced value. This is totally
     # fine!
@@ -399,7 +396,7 @@ def test_all_children_are_permitted_values(ir_type_and_kwargs):
         (2, integer_kw(0, 1), False),
         (10, integer_kw(0, 20), True),
         (int(2**128 / 2) - 1, integer_kw(), True),
-        (int(2**128 / 2), integer_kw(), False),
+        (int(2**128 / 2), integer_kw(), True),
         (math.nan, float_kw(0.0, 0.0), True),
         (math.nan, float_kw(0.0, 0.0, allow_nan=False), False),
         (2.0, float_kw(1.0, 3.0, smallest_nonzero_magnitude=2.5), False),
@@ -453,7 +450,7 @@ def test_choice_permitted(value, kwargs, permitted):
     assert choice_permitted(value, kwargs) == permitted
 
 
-@given(ir_nodes(was_forced=True))
+@given(nodes(was_forced=True))
 def test_forced_nodes_are_trivial(node):
     assert node.trivial
 
@@ -540,21 +537,17 @@ def test_forced_nodes_are_trivial(node):
             kwargs=integer_kw(max_value=10, shrink_towards=1),
             was_forced=False,
         ),
-        # TODO_IR: this *is* trivial by node.trivial, but not by shrinking, because
-        # the buffer ordering doesn't yet consider shrink_towards for unbounded
-        # integers this will be fixed (and this test case can be uncommented) when
-        # we move shrink ordering to the typed choice sequence.
-        # IRNode(
-        #     ir_type="integer",
-        #     value=1,
-        #     kwargs={
-        #         "min_value": None,
-        #         "max_value": None,
-        #         "weights": None,
-        #         "shrink_towards": 1,
-        #     },
-        #     was_forced=False,
-        # ),
+        IRNode(
+            ir_type="integer",
+            value=1,
+            kwargs={
+                "min_value": None,
+                "max_value": None,
+                "weights": None,
+                "shrink_towards": 1,
+            },
+            was_forced=False,
+        ),
     ],
 )
 def test_trivial_nodes(node):
@@ -675,29 +668,24 @@ def test_conservative_nontrivial_nodes(node):
     assert choice_equal(minimal(values()), node.value)
 
 
-@given(ir_nodes())
+@given(nodes())
 def test_ir_node_is_hashable(ir_node):
     hash(ir_node)
 
 
-@given(st.lists(ir_nodes()))
-def test_ir_size_positive(nodes):
-    assert ir_size(nodes) >= 0
+@given(st.lists(nodes()))
+def test_choices_size_positive(nodes):
+    assert choices_size([n.value for n in nodes]) >= 0
 
 
 @given(st.integers(min_value=1))
-def test_node_template_size(n):
-    node = NodeTemplate(type="simplest", size=n)
-    assert ir_size([node]) == n
-
-
-@given(st.lists(ir_nodes()), st.integers(min_value=0))
-def test_truncate_nodes(nodes, size):
-    assert len(truncate_choices_to_size(nodes, size)) <= len(nodes)
+def test_node_template_count(n):
+    node = ChoiceTemplate(type="simplest", count=n)
+    assert choice_count([node]) == n
 
 
 def test_node_template_to_overrun():
-    data = ConjectureData.for_choices([1, NodeTemplate("simplest", size=10)])
+    data = ConjectureData.for_choices([1, ChoiceTemplate("simplest", count=5)])
     data.draw_integer()
     with pytest.raises(StopTest):
         for _ in range(10):
@@ -708,22 +696,22 @@ def test_node_template_to_overrun():
 
 def test_node_template_single_node_overruns():
     # test for when drawing a single node takes more than BUFFER_SIZE, while in
-    # the NodeTemplate case
-    data = ConjectureData.for_choices((NodeTemplate("simplest", size=BUFFER_SIZE_IR),))
+    # the ChoiceTemplate case
+    data = ConjectureData.for_choices((ChoiceTemplate("simplest", count=1),))
     with pytest.raises(StopTest):
         data.draw_bytes(10_000, 10_000)
 
     assert data.status is Status.OVERRUN
 
 
-@given(ir_nodes())
+@given(nodes())
 def test_node_template_simplest_is_actually_trivial(node):
     # TODO_IR node.trivial is sound but not complete for floats.
     assume(node.ir_type != "float")
-    data = ConjectureData.for_choices((NodeTemplate("simplest", size=BUFFER_SIZE_IR),))
+    data = ConjectureData.for_choices((ChoiceTemplate("simplest", count=1),))
     getattr(data, f"draw_{node.ir_type}")(**node.kwargs)
-    assert len(data.ir_nodes) == 1
-    assert data.ir_nodes[0].trivial
+    assert len(data.nodes) == 1
+    assert data.nodes[0].trivial
 
 
 @given(ir_types_and_kwargs())
@@ -848,7 +836,7 @@ def test_integer_choice_index(kwargs, choices):
         assert choice_to_index(choice, kwargs) == i
 
 
-@given(st.lists(ir_nodes()))
+@given(st.lists(nodes()))
 def test_drawing_directly_matches_for_choices(nodes):
     data = ConjectureData.for_choices([n.value for n in nodes])
     for node in nodes:
@@ -900,3 +888,20 @@ def test_draw_directly_explicit():
 )
 def test_choices_key_distinguishes_weird_cases(choices1, choices2):
     assert choices_key(choices1) != choices_key(choices2)
+
+
+def test_node_template_overrun():
+    # different code path for overruning the ChoiceTemplate count, not BUFFER_SIZE.
+    cd = ConjectureData(
+        random=None,
+        prefix=[ChoiceTemplate("simplest", count=2)],
+        max_choices=100,
+    )
+
+    cd.draw_integer()
+    cd.draw_integer()
+    try:
+        cd.draw_integer()
+    except StopTest:
+        pass
+    assert cd.status is Status.OVERRUN
