@@ -30,8 +30,10 @@ from hypothesis.database import (
     InMemoryExampleDatabase,
     MultiplexedDatabase,
     ReadOnlyDatabase,
-    ir_from_bytes,
-    ir_to_bytes,
+    _pack_uleb128,
+    _unpack_uleb128,
+    choices_from_bytes,
+    choices_to_bytes,
 )
 from hypothesis.errors import HypothesisWarning
 from hypothesis.internal.compat import WINDOWS
@@ -41,7 +43,7 @@ from hypothesis.strategies import binary, lists, tuples
 from hypothesis.utils.conventions import not_set
 
 from tests.common.utils import skipif_emscripten
-from tests.conjecture.common import ir, ir_nodes
+from tests.conjecture.common import ir, nodes
 
 small_settings = settings(max_examples=50)
 
@@ -469,7 +471,7 @@ def test_background_write_database():
     assert set(db.fetch(b"a")) == {b"d"}
 
 
-@given(lists(ir_nodes()))
+@given(lists(nodes()))
 # covering examples
 @example(ir(True))
 @example(ir(1))
@@ -478,14 +480,23 @@ def test_background_write_database():
 @example(ir("a"))
 @example(ir(b"a"))
 @example(ir(b"a" * 50))
-def test_ir_nodes_roundtrips(nodes1):
-    s1 = ir_to_bytes([n.value for n in nodes1])
+@example(ir(b"1" * 100_000))  # really long bytes
+def test_nodes_roundtrips(nodes1):
+    s1 = choices_to_bytes([n.value for n in nodes1])
     assert isinstance(s1, bytes)
-    ir2 = ir_from_bytes(s1)
+    ir2 = choices_from_bytes(s1)
     assert len(nodes1) == len(ir2)
 
     for n1, v2 in zip(nodes1, ir2):
         assert choice_equal(n1.value, v2)
 
-    s2 = ir_to_bytes(ir2)
+    s2 = choices_to_bytes(ir2)
     assert s1 == s2
+
+
+@given(st.integers(min_value=0))
+def test_uleb_128_roundtrips(n1):
+    buffer1 = _pack_uleb128(n1)
+    idx, n2 = _unpack_uleb128(buffer1)
+    assert idx == len(buffer1)
+    assert n1 == n2
